<?php
/**
 * Created by PhpStorm.
 * User: smp
 * Date: 27/01/19
 * Time: 11:08 AM
 */

namespace Coordinadora;
use SoapClient;

class WebService
{

    const SANDBOX_URL_TRACKING_DISPATCHES = 'http://sandbox.coordinadora.com/ags/1.5/server.php?wsdl';

    const URL_TRACKING_DISPATCHES = 'https://ws.coordinadora.com/ags/1.5/server.php?wsdl';

    const SANDBOX_URL_GUIDES = 'http://sandbox.coordinadora.com/agw/ws/guias/1.6/server.php';

    const URL_GUIDES = 'http://guias.coordinadora.com/ws/guias/1.6/server.php';

    private string $_apikey;

    private string $_password_dispatches;

    private string $nit;

    private string $id_client;

    private string $user_guide;

    private string $password_guide;

    private static bool $sandbox = false;

    /**
     * WebService constructor.
     * @throws \Exception
     */
    public function __construct()
    {

        $i = func_num_args();

        if ($i !== 6)
            throw new \Exception("Invalid arguments");

        $this->_apikey = func_get_arg(0);
        $this->_password_dispatches = func_get_arg(1);
        $this->nit = func_get_arg(2);

        $this->id_client = func_get_arg(3);
        $this->user_guide = func_get_arg(4);
        $this->password_guide = func_get_arg(5);
    }

    public function sandbox_mode($status = false): static
    {
        if ($status){
            self::$sandbox = true;
        }
        return $this;
    }

    public static function get_url_tracking_dispatches(): string
    {
        if (self::$sandbox)
            return self::SANDBOX_URL_TRACKING_DISPATCHES;
        return self::URL_TRACKING_DISPATCHES;
    }

    public static function get_url_guides(): string
    {
        if (self::$sandbox)
            return self::SANDBOX_URL_GUIDES;
        return self::URL_GUIDES;
    }

    public static function url_soap($name_function): string
    {
        return self::isGuide($name_function) === false ? self::get_url_guides() : self::get_url_tracking_dispatches();
    }

    public static function isGuide($name_function): bool
    {
        return !str_contains($name_function, 'Guias');
    }

    protected function _access_connect_dispatches(): array
    {
        return array(
            'apikey' => $this->_apikey,
            'clave' => $this->_password_dispatches
        );
    }

    protected function _access_connect_guides(): array
    {
        return array(
            'id_cliente' => $this->id_client,
            'usuario' => $this->user_guide,
            'clave' => $this->encrypt_password($this->password_guide)
        );
    }

    /**
     * @return mixed
     * @throws \Exception
     */
    public static function Cotizador_departamentos(): mixed
    {
        return self::call_soap(__FUNCTION__);
    }

    /**
     * @return mixed
     * @throws \Exception
     */
    public static function Cotizador_ciudades(): mixed
    {
        return self::call_soap(__FUNCTION__);

    }

    /**
     * @param $params
     * @return mixed
     * @throws \Exception
     */
    public function Cotizador_cotizar($params): mixed
    {
        $body =  array(
            'p' => array_merge($params, array('nit'  => $this->nit), $this->_access_connect_dispatches()),
        );

        return $this->call_soap(__FUNCTION__, $body);
    }

    /**
     * @param $params
     * @return mixed
     * @throws \Exception
     */
    public function Recaudos_consultar($params): mixed
    {
        $body = array(
            'p' => array_merge($params, $this->_access_connect_dispatches()),
        );

        return $this->call_soap(__FUNCTION__, $body);
    }

    /**
     * @param $params
     * @return mixed
     * @throws \Exception
     */
    public function Recogidas_programar($params): mixed
    {
        $body = array(
            'p' => array_merge($params, $this->_access_connect_dispatches()),
        );

        return $this->call_soap(__FUNCTION__, $body);
    }

    /**
     * @param $params
     * @return mixed
     * @throws \Exception
     */
    public function Guias_generarGuia($params): mixed
    {
        $body = (object)array_merge($params, $this->_access_connect_guides());
        return $this->call_soap(__FUNCTION__, $body);
    }

    /**
     * @param $params
     * @return mixed
     * @throws \Exception
     */
    public function Guias_editarGuia($params): mixed
    {
        $body = (object)array_merge($params, $this->_access_connect_guides());
        return $this->call_soap(__FUNCTION__, $body);
    }

    /**
     * @param $params
     * @return mixed
     * @throws \Exception
     */
    public function Guias_generarDespacho($params): mixed
    {
        $body = (object)array_merge($params, $this->_access_connect_guides());
        return $this->call_soap(__FUNCTION__, $body);
    }

    /**
     * @param $params
     * @return mixed
     * @throws \Exception
     */
    public function Guias_anularGuia($params): mixed
    {
        $body = (object)array_merge($params, $this->_access_connect_guides());
        return $this->call_soap(__FUNCTION__, $body);
    }

    /**
     * @param $params
     * @return mixed
     * @throws \Exception
     */
    public function Guias_rastreoSimple($params): mixed
    {
        $body = (object)array_merge($params, $this->_access_connect_guides());
        return $this->call_soap(__FUNCTION__, $body);
    }

    /**
     * @param array $params
     * @return mixed
     * @throws \Exception
     */
    public function Guias_rastreoExtendido(array $params): mixed
    {
        $body = (object)array_merge($params, $this->_access_connect_guides());
        return $this->call_soap(__FUNCTION__, $body);
    }

    /**
     * @param $params
     * @return mixed
     * @throws \Exception
     */
    public function Guias_imprimirRotulos($params): mixed
    {
        $body = (object)array_merge($params, $this->_access_connect_guides());
        return $this->call_soap(__FUNCTION__, $body);
    }

    /**
     * @throws \Exception
     */
    private  static function call_soap($name_function, $params = array('p' => array())): mixed
    {
        try{
            if (self::isGuide($name_function) === false){
                $client = New SoapClient( null, array("location"  => self::url_soap($name_function),
                    "uri"         => self::url_soap($name_function),
                    "use"         => SOAP_LITERAL,
                    "trace"       => true,
                    "exceptions"  => true,
                    "soap_version"=> SOAP_1_2,
                    "connection_timeout"=> 30,
                    "encoding"=> "utf-8"));
            }else{
                $client = New SoapClient(self::url_soap($name_function), array(
                    "trace"       => true,
                    "soap_version"   => SOAP_1_2,
                    "connection_timeout"=> 30,
                    "encoding"=> "utf-8",
                    'stream_context' => stream_context_create(array(
                        'ssl' => array(
                            'verify_peer' => false,
                            'verify_peer_name' => false,
                            'allow_self_signed' => true
                        )
                    )),
                    'cache_wsdl' => WSDL_CACHE_NONE));
            }
            $data = $client->$name_function($params);
            $name_function_result = $name_function . "Result";
            return $data->$name_function_result ?? $data;
        }catch (\Exception $exception){
            throw new  \Exception($exception->getMessage());
        }
    }

    private function encrypt_password(string $password): string
    {
        return preg_match("/^([a-f0-9]{64})$/", $password) ? $password : hash('sha256', $password);
    }
}